"""
Pythagorean Spiral (Spiral of Theodorus)    (Press run ^)

Controls:
s - stop/start (you have to do this to make it start)
u - speed up
d - slow down
f - toggle max speed
c - toggle colors
h - show/hide cursor
r - reset
q - end
"""

import turtle
import time
import math
import random

def main():

    screen = turtle.Screen()
    spiral = Spiral(screen,
                    x=0,
                    y=0,
                    scale=30,
                    speed=5,
                    color=color,
                    color_on=False,
                    label=True,
                    hidden=True,
                    drawing=False,
    )

    screen.onkey(spiral.speed_up,
                "u")
                
    screen.onkey(spiral.slow_down,
                "d")
                
    screen.onkey(spiral.toggler("hidden", spiral.set_hide),
                "h")
                  
    screen.onkey(spiral.toggler("color_on"),
                "c")
                
    screen.onkey(spiral.toggler("drawing"),
                "s")
                
    screen.onkey(spiral.toggler("fast", spiral.set_fast),
                "f")
                
    screen.onkey(spiral.reset,
                "r")

    screen.onkey(spiral.finish,
                "q")            
    screen.listen()




def handleException(err):
    print(err)
    screen.bye()
    #p=input('Press any key, and then Choose Run, Python Shell.')
    #quit()
    
def loop_gen(lst):
  while True:
    for item in lst:
      yield item

black_only = loop_gen(["black"])
color = loop_gen(["red","blue","green",])

class Turtle(turtle.Turtle):
  def seth(self, angle):
    angle -= self.heading()
    angle %= 360
    if angle <= 180:
      self.left(angle)
    else:
      self.right(360 - angle)
  
  def point_to(self, x, y):
    self.seth(self.towards(x, y))

class Spiral():
  def __init__(self, screen, x=0, y=0, scale=10, speed=5,
              label=True, color=black_only, color_on=False, hidden=True,
              drawing=True):
    self.screen = screen
    
    self.x = x
    self.y = y
    
    self.scale = scale
    self.speed = speed
    self.label = label
    self.color = color
    
    self.color_on = color_on
    self.hidden = hidden
    self.drawing = drawing
    
    self.n = 1
    self.start_reset = False
    self.fast = False
    
    self.turtle = Turtle()
    self._init_turtle()
    
    self.draw_triangle()
  
  def _init_turtle(self):
    self.set_hide()
    self.turtle.speed(0)
    self.turtle.left(180)
    self.turtle.pu()
    self.turtle.goto(self.x - self.scale, self.y)
    self.turtle.pd()
    self.set_speed()
  
  def get_color(self):
    if self.color_on:
      col=["red","blue","green"]
      col_pick = random.randint(1,3)-1
      #print(col_pick)
      return col[col_pick]
      #return self.color.next()
    else:
      return "black"
  
  def draw_triangle(self):
    if self.drawing:
      dist = math.sqrt(self.n) * self.scale
      
      self.turtle.pencolor(self.get_color())
      self.turtle.point_to(self.x, self.y)
      self.turtle.fd(dist)
      self.turtle.bk(dist - self.scale / 3)
      
      if self.label:
        self.turtle.pu()
        self.turtle.left(90)
        self.turtle.fd(self.scale / 3)
        self.turtle.write(u"√"+str(self.n),
                          align="center",
                          font=("Arial", int(self.scale / 4), "normal"))
        self.turtle.bk(self.scale / 3)
        self.turtle.right(90)
        self.turtle.pd()
  
      self.turtle.bk(self.scale / 3)
      self.turtle.lt(90)
      self.turtle.fd(self.scale)
      self.n += 1
    
    if self.start_reset:
      self._reset()
      self.start_reset = False
    
    self.screen.ontimer(self.draw_triangle, 0)
  
  def set_fast(self):
    if self.fast:
      self.turtle.speed(0)
    else:
      self.set_speed()
  
  def cap_speed(self):
    if self.speed < 1:
      self.speed = 1
    elif self.speed > 10:
      self.speed = 10
  
  def set_speed(self):
    self.cap_speed()
    if not self.fast:
      self.turtle.speed(self.speed)
  
  def set_hide(self):
    if self.hidden:
      self.turtle.ht()
    else:
      self.turtle.st()
  
  def speed_up(self):
    self.speed += 1
    self.set_speed()
  
  def slow_down(self):
    self.speed -= 1
    self.set_speed()
  
  def toggle(self, attr):
    print("Toggling", attr)
    setattr(self, attr, not getattr(self, attr))
  
  def toggler(self, attr, after=None):
    def inner():
      self.toggle(attr)
      if after:
        after()
    return inner
  
  def _reset(self):
    self.drawing = False
    self.n = 1
    self.turtle.reset()
    self._init_turtle()

  def reset(self):
    self.start_reset = True

  def finish(self):
    print("Exiting program...")
    exit
    try:
        turtle.bye()
    except:
        err='Bye...'
        handleException(err)
        
if __name__ == "__main__":
    main()
